/* ************************************************************************
*   File: class.c                                       Part of CircleMUD *
*  Usage: Source file for class-specific code                             *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

/*
 * This file attempts to concentrate most of the code which must be changed
 * in order for new classes to be added.  If you're adding a new class,
 * you should go through this entire file from beginning to end and add
 * the appropriate new special cases for your new class.
 */



#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "db.h"
#include "utils.h"
#include "spells.h"
#include "interpreter.h"
#include "constants.h"

extern int siteok_everyone;

/* local functions */
int parse_class(char arg);
long find_class_bitvector(char arg);
byte saving_throws(int class_num, int type, int level);
int thaco(int class_num, int level);
void roll_real_abils(struct char_data * ch);
void do_start(struct char_data * ch);
int backstab_mult(int level);
int invalid_class(struct char_data *ch, struct obj_data *obj);
int level_exp(int chclass, int level);
const char *title_male(int chclass, int level);
const char *title_female(int chclass, int level);

/* Names first */

const char *class_abbrevs[] = {
  "Mu",
  "Cl",
  "Th",
  "Wa",
  "\n"
};


const char *pc_class_types[] = {
  "Magic User",
  "Cleric",
  "Thief",
  "Warrior",
  "\n"
};


/* The menu for choosing a class in interpreter.c: */
const char *class_menu =
"\r\n"
"Select a class:\r\n"
"  [C]leric\r\n"
"  [T]hief\r\n"
"  [W]arrior\r\n"
"  [M]agic-user\r\n";



/*
 * The code to interpret a class letter -- used in interpreter.c when a
 * new character is selecting a class and by 'set class' in act.wizard.c.
 */

int parse_class(char arg)
{
  arg = LOWER(arg);

  switch (arg) {
  case 'm': return CLASS_MAGIC_USER;
  case 'c': return CLASS_CLERIC;
  case 'w': return CLASS_WARRIOR;
  case 't': return CLASS_THIEF;
  default:  return CLASS_UNDEFINED;
  }
}

/*
 * bitvectors (i.e., powers of two) for each class, mainly for use in
 * do_who and do_users.  Add new classes at the end so that all classes
 * use sequential powers of two (1 << 0, 1 << 1, 1 << 2, 1 << 3, 1 << 4,
 * 1 << 5, etc.
 */

long find_class_bitvector(char arg)
{
  arg = LOWER(arg);

  switch (arg) {
    case 'm': return (1 << CLASS_MAGIC_USER);
    case 'c': return (1 << CLASS_CLERIC);
    case 't': return (1 << CLASS_THIEF);
    case 'w': return (1 << CLASS_WARRIOR);
    default:  return 0;
  }
}


/*
 * These are definitions which control the guildmasters for each class.
 *
 * The first field (top line) controls the highest percentage skill level
 * a character of the class is allowed to attain in any skill.  (After
 * this level, attempts to practice will say "You are already learned in
 * this area."
 * 
 * The second line controls the maximum percent gain in learnedness a
 * character is allowed per practice -- in other words, if the random
 * die throw comes out higher than this number, the gain will only be
 * this number instead.
 *
 * The third line controls the minimu percent gain in learnedness a
 * character is allowed per practice -- in other words, if the random
 * die throw comes out below this number, the gain will be set up to
 * this number.
 * 
 * The fourth line simply sets whether the character knows 'spells'
 * or 'skills'.  This does not affect anything except the message given
 * to the character when trying to practice (i.e. "You know of the
 * following spells" vs. "You know of the following skills"
 */

#define SPELL	0
#define SKILL	1

/* #define LEARNED_LEVEL	0  % known which is considered "learned" */
/* #define MAX_PER_PRAC		1  max percent gain in skill per practice */
/* #define MIN_PER_PRAC		2  min percent gain in skill per practice */
/* #define PRAC_TYPE		3  should it say 'spell' or 'skill'?	*/

int prac_params[4][NUM_CLASSES] = {
  /* MAG	CLE	THE	WAR */
  {95,		95,	85,	80},		/* learned level */
  {100,		100,	12,	12},		/* max per prac */
  {25,		25,	0,	0,},		/* min per pac */
  {SPELL,	SPELL,	SKILL,	SKILL}		/* prac name */
};


/*
 * ...And the appropriate rooms for each guildmaster/guildguard; controls
 * which types of people the various guildguards let through.  i.e., the
 * first line shows that from room 3017, only MAGIC_USERS are allowed
 * to go south.
 *
 * Don't forget to visit spec_assign.c if you create any new mobiles that
 * should be a guild master or guard so they can act appropriately. If you
 * "recycle" the existing mobs that are used in other guilds for your new
 * guild, then you don't have to change that file, only here.
 */
int guild_info[][3] = {

/* Midgaard */
  {CLASS_MAGIC_USER,	3017,	SCMD_SOUTH},
  {CLASS_CLERIC,	3004,	SCMD_NORTH},
  {CLASS_THIEF,		3027,	SCMD_EAST},
  {CLASS_WARRIOR,	3021,	SCMD_EAST},

/* Brass Dragon */
  {-999 /* all */ ,	5065,	SCMD_WEST},

/* this must go last -- add new guards above! */
  {-1, -1, -1}
};



/*
 * Saving throws for:
 * MCTW
 *   PARA, ROD, PETRI, BREATH, SPELL
 *     Levels 0-95
 *
 * Do not forget to change extern declaration in magic.c if you add to this.
 */

byte saving_throws(int class_num, int type, int level)
{
  switch (class_num) {
  case CLASS_MAGIC_USER:
    switch (type) {
    case SAVING_PARA:	/* Paralyzation */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for mage paralyzation saving throw.");
	break;
      }
    case SAVING_ROD:	/* Rods */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for mage rod saving throw.");
	break;
      }
    case SAVING_PETRI:	/* Petrification */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for mage petrification saving throw.");
	break;
      }
    case SAVING_BREATH:	/* Breath weapons */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for mage breath saving throw.");
	break;
      }
    case SAVING_SPELL:	/* Generic spells */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for mage spell saving throw.");
	break;
      }
    default:
      log("SYSERR: Invalid saving throw type.");
      break;
    }
    break;
  case CLASS_CLERIC:
    switch (type) {
    case SAVING_PARA:	/* Paralyzation */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for cleric paralyzation saving throw.");
	break;
      }
    case SAVING_ROD:	/* Rods */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for cleric rod saving throw.");
	break;
      }
    case SAVING_PETRI:	/* Petrification */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for cleric petrification saving throw.");
	break;
      }
    case SAVING_BREATH:	/* Breath weapons */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for cleric breath saving throw.");
	break;
      }
    case SAVING_SPELL:	/* Generic spells */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for cleric spell saving throw.");
	break;
      }
    default:
      log("SYSERR: Invalid saving throw type.");
      break;
    }
    break;
  case CLASS_THIEF:
    switch (type) {
    case SAVING_PARA:	/* Paralyzation */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for thief paralyzation saving throw.");
	break;
      }
    case SAVING_ROD:	/* Rods */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for thief rod saving throw.");
	break;
      }
    case SAVING_PETRI:	/* Petrification */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for thief petrification saving throw.");
	break;
      }
    case SAVING_BREATH:	/* Breath weapons */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for thief breath saving throw.");
	break;
      }
    case SAVING_SPELL:	/* Generic spells */
      switch (level) {
      case  0: return 90;
      case  1: return 70;
      case  2: return 69;
      case  3: return 68;
      case  4: return 67;
      case  5: return 66;
      case  6: return 65;
      case  7: return 64;
      case  8: return 63;
      case  9: return 62;
      case 10: return 61;
      case 11: return 60;
      case 12: return 59;
      case 13: return 58;
      case 14: return 57;
      case 15: return 56;
      case 16: return 55;
      case 17: return 54;
      case 18: return 53;
      case 19: return 52;
      case 20: return 51;
      case 21: return 50;
      case 22: return 49;
      case 23: return 48;
      case 24: return 47;
      case 25: return 46;
      case 26: return 45;
      case 27: return 44;
      case 28: return 43;
      case 29: return 42;
      case 30: return 41;
      case 31: return 40;
      case 32: return 39;
      case 33: return  38;
      case 34: return  37;
      case 35: return  36;
      case 36: return  35;
      case 37: return  34;
      case 38: return  33;
      case 39: return  32;
      case 40: return  31;
      case 41: return  30;
      case 42: return  29;
      case 43: return  28;
      case 44: return  27;
      case 45: return  26;
      case 46: return  25;
      case 47: return  24;
      case 48: return  23;
      case 49: return  22;
      case 50: return  21;
      case 51: return  20;
      case 52: return  19;
      case 53: return  18;
      case 54: return  17;
      case 55: return  16;
      case 56: return  15;
      case 57: return  14;
      case 58: return  13;
      case 59: return  12;
      case 60: return  11;
      case 61: return  10;
      case 62: return  10;
      case 63: return  10;
      case 64: return  10;
      case 65: return  10;
      case 66: return  10;
      case 67: return  10;
      case 68: return  10;
      case 69: return  10;
      case 70: return  10;
      case 71: return  10;
      case 72: return  10;
      case 73: return  10;
      case 74: return  10;
      case 75: return  10;
      case 76: return  10;
      case 77: return  10;
      case 78: return  10;
      case 79: return  10;
      case 80: return  10;
      case 81: return  10;
      case 82: return  10;
      case 83: return  10;
      case 84: return  10;
      case 85: return  10;
      case 86: return  10;
      case 87: return  10;
      case 88: return  10;
      case 89: return  10;
      case 90: return  10;
      case 91: return  10;
      case 92: return  10;
      case 93: return  5;
      case 94: return  5;
      case 95: return  5;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for thief spell saving throw.");
	break;
      }
    default:
      log("SYSERR: Invalid saving throw type.");
      break;
    }
    break;
  case CLASS_WARRIOR:
    switch (type) {
    case SAVING_PARA:	/* Paralyzation */
      switch (level) {
      case  0: return 90;
      case  1: return 85;
      case  2: return 84;
      case  3: return 83;
      case  4: return 82;
      case  5: return 81;
      case  6: return 80;
      case  7: return 79;
      case  8: return 78;
      case  9: return 77;
      case 10: return 76;
      case 11: return 75;
      case 12: return 74;
      case 13: return 74;
      case 14: return 73;
      case 15: return 72;
      case 16: return 72;
      case 17: return 71;
      case 18: return 71;
      case 19: return 70;
      case 20: return 70;
      case 21: return 70;
      case 22: return 69;
      case 23: return 69;
      case 24: return 69;
      case 25: return 68;
      case 26: return 68;
      case 27: return 68;
      case 28: return 67;
      case 29: return 67;
      case 30: return 67;
      case 31: return 66;
      case 32: return 66;
      case 33: return  66;
      case 34: return  65;
      case 35: return  65;
      case 36: return  65;
      case 37: return  64;
      case 38: return  64;
      case 39: return  64;
      case 40: return  63;
      case 41: return  63;	/* Some mobiles. */
      case 42: return  63;
      case 43: return  62;
      case 44: return  62;
      case 45: return  62;
      case 46: return  61;
      case 47: return  61;
      case 48: return  61;
      case 49: return  60;
      case 50: return  60;
      case 51: return  60;
      case 52: return  59;
      case 53: return  59;
      case 54: return  58;
      case 55: return  57;
      case 56: return  56;
      case 57: return  55;
      case 58: return  54;
      case 59: return  53;
      case 60: return  52;
      case 61: return  51;
      case 62: return  50;
      case 63: return  48;
      case 64: return  46;
      case 65: return  45;
      case 66: return  43;
      case 67: return  41;
      case 68: return  40;
      case 69: return  39;
      case 70: return  37;
      case 71: return  36;
      case 72: return  35;
      case 73: return  34;
      case 74: return  33;
      case 75: return  32;
      case 76: return  31;
      case 77: return  30;
      case 78: return  29;
      case 79: return  27;
      case 80: return  26;
      case 81: return  25;
      case 82: return  24;
      case 83: return  23;
      case 84: return  22;
      case 85: return  21;
      case 86: return  20;
      case 87: return  19;
      case 88: return  17;
      case 89: return  15;
      case 90: return  14;
      case 91: return  13;
      case 92: return  12;
      case 93: return  11;
      case 94: return  10;
      case 95: return  10;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;

      default:
	log("SYSERR: Missing level for warrior paralyzation saving throw.");
	break;	
      }
    case SAVING_ROD:	/* Rods */
      switch (level) {
      case  0: return 90;
      case  1: return 85;
      case  2: return 84;
      case  3: return 83;
      case  4: return 82;
      case  5: return 81;
      case  6: return 80;
      case  7: return 79;
      case  8: return 78;
      case  9: return 77;
      case 10: return 76;
      case 11: return 75;
      case 12: return 74;
      case 13: return 74;
      case 14: return 73;
      case 15: return 72;
      case 16: return 72;
      case 17: return 71;
      case 18: return 71;
      case 19: return 70;
      case 20: return 70;
      case 21: return 70;
      case 22: return 69;
      case 23: return 69;
      case 24: return 69;
      case 25: return 68;
      case 26: return 68;
      case 27: return 68;
      case 28: return 67;
      case 29: return 67;
      case 30: return 67;
      case 31: return 66;
      case 32: return 66;
      case 33: return  66;
      case 34: return  65;
      case 35: return  65;
      case 36: return  65;
      case 37: return  64;
      case 38: return  64;
      case 39: return  64;
      case 40: return  63;
      case 41: return  63;	/* Some mobiles. */
      case 42: return  63;
      case 43: return  62;
      case 44: return  62;
      case 45: return  62;
      case 46: return  61;
      case 47: return  61;
      case 48: return  61;
      case 49: return  60;
      case 50: return  60;
      case 51: return  60;
      case 52: return  59;
      case 53: return  59;
      case 54: return  58;
      case 55: return  57;
      case 56: return  56;
      case 57: return  55;
      case 58: return  54;
      case 59: return  53;
      case 60: return  52;
      case 61: return  51;
      case 62: return  50;
      case 63: return  48;
      case 64: return  46;
      case 65: return  45;
      case 66: return  43;
      case 67: return  41;
      case 68: return  40;
      case 69: return  39;
      case 70: return  37;
      case 71: return  36;
      case 72: return  35;
      case 73: return  34;
      case 74: return  33;
      case 75: return  32;
      case 76: return  31;
      case 77: return  30;
      case 78: return  29;
      case 79: return  27;
      case 80: return  26;
      case 81: return  25;
      case 82: return  24;
      case 83: return  23;
      case 84: return  22;
      case 85: return  21;
      case 86: return  20;
      case 87: return  19;
      case 88: return  17;
      case 89: return  15;
      case 90: return  14;
      case 91: return  13;
      case 92: return  12;
      case 93: return  11;
      case 94: return  10;
      case 95: return  10;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for warrior rod saving throw.");
	break;
      }
    case SAVING_PETRI:	/* Petrification */
      switch (level) {
      case  0: return 90;
      case  1: return 85;
      case  2: return 84;
      case  3: return 83;
      case  4: return 82;
      case  5: return 81;
      case  6: return 80;
      case  7: return 79;
      case  8: return 78;
      case  9: return 77;
      case 10: return 76;
      case 11: return 75;
      case 12: return 74;
      case 13: return 74;
      case 14: return 73;
      case 15: return 72;
      case 16: return 72;
      case 17: return 71;
      case 18: return 71;
      case 19: return 70;
      case 20: return 70;
      case 21: return 70;
      case 22: return 69;
      case 23: return 69;
      case 24: return 69;
      case 25: return 68;
      case 26: return 68;
      case 27: return 68;
      case 28: return 67;
      case 29: return 67;
      case 30: return 67;
      case 31: return 66;
      case 32: return 66;
      case 33: return  66;
      case 34: return  65;
      case 35: return  65;
      case 36: return  65;
      case 37: return  64;
      case 38: return  64;
      case 39: return  64;
      case 40: return  63;
      case 41: return  63;	/* Some mobiles. */
      case 42: return  63;
      case 43: return  62;
      case 44: return  62;
      case 45: return  62;
      case 46: return  61;
      case 47: return  61;
      case 48: return  61;
      case 49: return  60;
      case 50: return  60;
      case 51: return  60;
      case 52: return  59;
      case 53: return  59;
      case 54: return  58;
      case 55: return  57;
      case 56: return  56;
      case 57: return  55;
      case 58: return  54;
      case 59: return  53;
      case 60: return  52;
      case 61: return  51;
      case 62: return  50;
      case 63: return  48;
      case 64: return  46;
      case 65: return  45;
      case 66: return  43;
      case 67: return  41;
      case 68: return  40;
      case 69: return  39;
      case 70: return  37;
      case 71: return  36;
      case 72: return  35;
      case 73: return  34;
      case 74: return  33;
      case 75: return  32;
      case 76: return  31;
      case 77: return  30;
      case 78: return  29;
      case 79: return  27;
      case 80: return  26;
      case 81: return  25;
      case 82: return  24;
      case 83: return  23;
      case 84: return  22;
      case 85: return  21;
      case 86: return  20;
      case 87: return  19;
      case 88: return  17;
      case 89: return  15;
      case 90: return  14;
      case 91: return  13;
      case 92: return  12;
      case 93: return  11;
      case 94: return  10;
      case 95: return  10;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for warrior petrification saving throw.");
	break;
      }
    case SAVING_BREATH:	/* Breath weapons */
      switch (level) {
      case  0: return 90;
      case  1: return 85;
      case  2: return 84;
      case  3: return 83;
      case  4: return 82;
      case  5: return 81;
      case  6: return 80;
      case  7: return 79;
      case  8: return 78;
      case  9: return 77;
      case 10: return 76;
      case 11: return 75;
      case 12: return 74;
      case 13: return 74;
      case 14: return 73;
      case 15: return 72;
      case 16: return 72;
      case 17: return 71;
      case 18: return 71;
      case 19: return 70;
      case 20: return 70;
      case 21: return 70;
      case 22: return 69;
      case 23: return 69;
      case 24: return 69;
      case 25: return 68;
      case 26: return 68;
      case 27: return 68;
      case 28: return 67;
      case 29: return 67;
      case 30: return 67;
      case 31: return 66;
      case 32: return 66;
      case 33: return  66;
      case 34: return  65;
      case 35: return  65;
      case 36: return  65;
      case 37: return  64;
      case 38: return  64;
      case 39: return  64;
      case 40: return  63;
      case 41: return  63;	/* Some mobiles. */
      case 42: return  63;
      case 43: return  62;
      case 44: return  62;
      case 45: return  62;
      case 46: return  61;
      case 47: return  61;
      case 48: return  61;
      case 49: return  60;
      case 50: return  60;
      case 51: return  60;
      case 52: return  59;
      case 53: return  59;
      case 54: return  58;
      case 55: return  57;
      case 56: return  56;
      case 57: return  55;
      case 58: return  54;
      case 59: return  53;
      case 60: return  52;
      case 61: return  51;
      case 62: return  50;
      case 63: return  48;
      case 64: return  46;
      case 65: return  45;
      case 66: return  43;
      case 67: return  41;
      case 68: return  40;
      case 69: return  39;
      case 70: return  37;
      case 71: return  36;
      case 72: return  35;
      case 73: return  34;
      case 74: return  33;
      case 75: return  32;
      case 76: return  31;
      case 77: return  30;
      case 78: return  29;
      case 79: return  27;
      case 80: return  26;
      case 81: return  25;
      case 82: return  24;
      case 83: return  23;
      case 84: return  22;
      case 85: return  21;
      case 86: return  20;
      case 87: return  19;
      case 88: return  17;
      case 89: return  15;
      case 90: return  14;
      case 91: return  13;
      case 92: return  12;
      case 93: return  11;
      case 94: return  10;
      case 95: return  10;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for warrior breath saving throw.");
	break;
      }
    case SAVING_SPELL:	/* Generic spells */
      switch (level) {
      case  0: return 90;
      case  1: return 85;
      case  2: return 84;
      case  3: return 83;
      case  4: return 82;
      case  5: return 81;
      case  6: return 80;
      case  7: return 79;
      case  8: return 78;
      case  9: return 77;
      case 10: return 76;
      case 11: return 75;
      case 12: return 74;
      case 13: return 74;
      case 14: return 73;
      case 15: return 72;
      case 16: return 72;
      case 17: return 71;
      case 18: return 71;
      case 19: return 70;
      case 20: return 70;
      case 21: return 70;
      case 22: return 69;
      case 23: return 69;
      case 24: return 69;
      case 25: return 68;
      case 26: return 68;
      case 27: return 68;
      case 28: return 67;
      case 29: return 67;
      case 30: return 67;
      case 31: return 66;
      case 32: return 66;
      case 33: return  66;
      case 34: return  65;
      case 35: return  65;
      case 36: return  65;
      case 37: return  64;
      case 38: return  64;
      case 39: return  64;
      case 40: return  63;
      case 41: return  63;	/* Some mobiles. */
      case 42: return  63;
      case 43: return  62;
      case 44: return  62;
      case 45: return  62;
      case 46: return  61;
      case 47: return  61;
      case 48: return  61;
      case 49: return  60;
      case 50: return  60;
      case 51: return  60;
      case 52: return  59;
      case 53: return  59;
      case 54: return  58;
      case 55: return  57;
      case 56: return  56;
      case 57: return  55;
      case 58: return  54;
      case 59: return  53;
      case 60: return  52;
      case 61: return  51;
      case 62: return  50;
      case 63: return  48;
      case 64: return  46;
      case 65: return  45;
      case 66: return  43;
      case 67: return  41;
      case 68: return  40;
      case 69: return  39;
      case 70: return  37;
      case 71: return  36;
      case 72: return  35;
      case 73: return  34;
      case 74: return  33;
      case 75: return  32;
      case 76: return  31;
      case 77: return  30;
      case 78: return  29;
      case 79: return  27;
      case 80: return  26;
      case 81: return  25;
      case 82: return  24;
      case 83: return  23;
      case 84: return  22;
      case 85: return  21;
      case 86: return  20;
      case 87: return  19;
      case 88: return  17;
      case 89: return  15;
      case 90: return  14;
      case 91: return  13;
      case 92: return  12;
      case 93: return  11;
      case 94: return  10;
      case 95: return  10;
      case 96: return  0;
      case 97: return  0;
      case 98: return  0;
      case 99: return  0;
      default:
	log("SYSERR: Missing level for warrior spell saving throw.");
	break;
      }
    default:
      log("SYSERR: Invalid saving throw type.");
      break;
    }
  default:
    log("SYSERR: Invalid class saving throw.");
    break;
  }

  /* Should not get here unless something is wrong. */
  return 100;
}

/* THAC0 for classes and levels.  (To Hit Armor Class 0) */
int thaco(int class_num, int level)
{
  switch (class_num) {
  case CLASS_MAGIC_USER:
    switch (level) {
case 0: return 100;
case 1: return 20;
case 2: return 20;
case 3: return 20;
case 4: return 20;
case 5: return 20;
case 6: return 19;
case 7: return 19;
case 8: return 19;  
case 9: return 19;
case 10: return 19;
case 11: return 18;
case 12: return 18;
case 13: return 18;
case 14: return 18;
case 15: return 18;
case 16: return 17;
case 17: return 17;
case 18: return 17;
case 19: return 17;
case 20: return 17;
case 21: return 16;
case 22: return 16;
case 23: return 16;
case 24: return 16;
case 25: return 16;
case 26: return 15;
case 27: return 15;
case 28: return 15;
case 29: return 15;
case 30: return 15;
case 31: return 14;
case 32: return 14;
case 33: return 14;
case 34: return 14; 
case 35: return 14;
case 36: return 13;
case 37: return 13;
case 38: return 13;
case 39: return 13;
case 40: return 13;
case 41: return 12;
case 42: return 12;
case 43: return 12;
case 44: return 12;
case 45: return 12;
case 46: return 11;
case 47: return 11;
case 48: return 11;
case 49: return 11;
case 50: return 11;
case 51: return 10;
case 52: return 10;
case 53: return 10;
case 54: return 10;
case 55: return 10;
case 56: return 9; 
case 57: return 9; 
case 58: return 9; 
case 59: return 9; 
case 60: return 9;  
case 61: return 8; 
case 62: return 8; 
case 63: return 8; 
case 64: return 8; 
case 65: return 8; 
case 66: return 7; 
case 67: return 7; 
case 68: return 7; 
case 69: return 7; 
case 70: return 7; 
case 71: return 6; 
case 72: return 6; 
case 73: return 6; 
case 74: return 6; 
case 75: return 6; 
case 76: return 5; 
case 77: return 5;
case 78: return 5; 
case 79: return 5; 
case 80: return 5; 
case 81: return 4; 
case 82: return 4; 
case 83: return 4; 
case 84: return 4; 
case 85: return 4; 
case 86: return 3;  
case 87: return 3; 
case 88: return 3; 
case 89: return 3; 
case 90: return 3; 
case 91: return 2; 
case 92: return 2; 
case 93: return 2; 
case 94: return 2; 
case 95: return 2; 
case 96: return 1; 
case 97: return 1; 
case 98: return 1; 
case 99: return 1;
    default:
      log("SYSERR: Missing level for mage thac0.");
    }
  case CLASS_CLERIC:
    switch (level) {
case 0: return 100;
case 1: return 20;
case 2: return 20;
case 3: return 20;
case 4: return 20;
case 5: return 20;
case 6: return 19;
case 7: return 19;
case 8: return 19;  
case 9: return 19;
case 10: return 19;
case 11: return 18;
case 12: return 18;
case 13: return 18;
case 14: return 18;
case 15: return 18;
case 16: return 17;
case 17: return 17;
case 18: return 17;
case 19: return 17;
case 20: return 17;
case 21: return 16;
case 22: return 16;
case 23: return 16;
case 24: return 16;
case 25: return 16;
case 26: return 15;
case 27: return 15;
case 28: return 15;
case 29: return 15;
case 30: return 15;
case 31: return 14;
case 32: return 14;
case 33: return 14;
case 34: return 14; 
case 35: return 14;
case 36: return 13;
case 37: return 13;
case 38: return 13;
case 39: return 13;
case 40: return 13;
case 41: return 12;
case 42: return 12;
case 43: return 12;
case 44: return 12;
case 45: return 12;
case 46: return 11;
case 47: return 11;
case 48: return 11;
case 49: return 11;
case 50: return 11;
case 51: return 10;
case 52: return 10;
case 53: return 10;
case 54: return 10;
case 55: return 10;
case 56: return 9; 
case 57: return 9; 
case 58: return 9; 
case 59: return 9; 
case 60: return 9;  
case 61: return 8; 
case 62: return 8; 
case 63: return 8; 
case 64: return 8; 
case 65: return 8; 
case 66: return 7; 
case 67: return 7; 
case 68: return 7; 
case 69: return 7; 
case 70: return 7; 
case 71: return 6; 
case 72: return 6; 
case 73: return 6; 
case 74: return 6; 
case 75: return 6; 
case 76: return 5; 
case 77: return 5;
case 78: return 5; 
case 79: return 5; 
case 80: return 5; 
case 81: return 4; 
case 82: return 4; 
case 83: return 4; 
case 84: return 4; 
case 85: return 4; 
case 86: return 3;  
case 87: return 3; 
case 88: return 3; 
case 89: return 3; 
case 90: return 3; 
case 91: return 2; 
case 92: return 2; 
case 93: return 2; 
case 94: return 2; 
case 95: return 2; 
case 96: return 1; 
case 97: return 1; 
case 98: return 1; 
case 99: return 1; 
    default:
      log("SYSERR: Missing level for cleric thac0.");
    }
  case CLASS_THIEF:
    switch (level) {
case 0: return 100;
case 1: return 20;
case 2: return 20;
case 3: return 20;
case 4: return 20;
case 5: return 20;
case 6: return 19;
case 7: return 19;
case 8: return 19;  
case 9: return 19;
case 10: return 19;
case 11: return 18;
case 12: return 18;
case 13: return 18;
case 14: return 18;
case 15: return 18;
case 16: return 17;
case 17: return 17;
case 18: return 17;
case 19: return 17;
case 20: return 17;
case 21: return 16;
case 22: return 16;
case 23: return 16;
case 24: return 16;
case 25: return 16;
case 26: return 15;
case 27: return 15;
case 28: return 15;
case 29: return 15;
case 30: return 15;
case 31: return 14;
case 32: return 14;
case 33: return 14;
case 34: return 14; 
case 35: return 14;
case 36: return 13;
case 37: return 13;
case 38: return 13;
case 39: return 13;
case 40: return 13;
case 41: return 12;
case 42: return 12;
case 43: return 12;
case 44: return 12;
case 45: return 12;
case 46: return 11;
case 47: return 11;
case 48: return 11;
case 49: return 11;
case 50: return 11;
case 51: return 10;
case 52: return 10;
case 53: return 10;
case 54: return 10;
case 55: return 10;
case 56: return 9; 
case 57: return 9; 
case 58: return 9; 
case 59: return 9; 
case 60: return 9;  
case 61: return 8; 
case 62: return 8; 
case 63: return 8; 
case 64: return 8; 
case 65: return 8; 
case 66: return 7; 
case 67: return 7; 
case 68: return 7; 
case 69: return 7; 
case 70: return 7; 
case 71: return 6; 
case 72: return 6; 
case 73: return 6; 
case 74: return 6; 
case 75: return 6; 
case 76: return 5; 
case 77: return 5;
case 78: return 5; 
case 79: return 5; 
case 80: return 5; 
case 81: return 4; 
case 82: return 4; 
case 83: return 4; 
case 84: return 4; 
case 85: return 4; 
case 86: return 3;  
case 87: return 3; 
case 88: return 3; 
case 89: return 3; 
case 90: return 3; 
case 91: return 2; 
case 92: return 2; 
case 93: return 2; 
case 94: return 2; 
case 95: return 2; 
case 96: return 1; 
case 97: return 1; 
case 98: return 1; 
case 99: return 1; 
    default:
      log("SYSERR: Missing level for thief thac0.");
    }
  case CLASS_WARRIOR:
    switch (level) {
case 0: return 100;
case 1: return 20;
case 2: return 20;
case 3: return 20;
case 4: return 20;
case 5: return 20;
case 6: return 19;
case 7: return 19;
case 8: return 19;  
case 9: return 19;
case 10: return 19;
case 11: return 18;
case 12: return 18;
case 13: return 18;
case 14: return 18;
case 15: return 18;
case 16: return 17;
case 17: return 17;
case 18: return 17;
case 19: return 17;
case 20: return 17;
case 21: return 16;
case 22: return 16;
case 23: return 16;
case 24: return 16;
case 25: return 16;
case 26: return 15;
case 27: return 15;
case 28: return 15;
case 29: return 15;
case 30: return 15;
case 31: return 14;
case 32: return 14;
case 33: return 14;
case 34: return 14; 
case 35: return 14;
case 36: return 13;
case 37: return 13;
case 38: return 13;
case 39: return 13;
case 40: return 13;
case 41: return 12;
case 42: return 12;
case 43: return 12;
case 44: return 12;
case 45: return 12;
case 46: return 11;
case 47: return 11;
case 48: return 11;
case 49: return 11;
case 50: return 11;
case 51: return 10;
case 52: return 10;
case 53: return 10;
case 54: return 10;
case 55: return 10;
case 56: return 9; 
case 57: return 9; 
case 58: return 9; 
case 59: return 9; 
case 60: return 9;  
case 61: return 8; 
case 62: return 8; 
case 63: return 8; 
case 64: return 8; 
case 65: return 8; 
case 66: return 7; 
case 67: return 7; 
case 68: return 7; 
case 69: return 7; 
case 70: return 7; 
case 71: return 6; 
case 72: return 6; 
case 73: return 6; 
case 74: return 6; 
case 75: return 6; 
case 76: return 5; 
case 77: return 5;
case 78: return 5; 
case 79: return 5; 
case 80: return 5; 
case 81: return 4; 
case 82: return 4; 
case 83: return 4; 
case 84: return 4; 
case 85: return 4; 
case 86: return 3;  
case 87: return 3; 
case 88: return 3; 
case 89: return 3; 
case 90: return 3; 
case 91: return 2; 
case 92: return 2; 
case 93: return 2; 
case 94: return 2; 
case 95: return 2; 
case 96: return 1; 
case 97: return 1; 
case 98: return 1; 
case 99: return 1; 
    default:
      log("SYSERR: Missing level for warrior thac0.");
    }
  default:
    log("SYSERR: Unknown class in thac0 chart.");
  }

  /* Will not get there unless something is wrong. */
  return 100;
}


/*
 * Roll the 6 stats for a character... each stat is made of the sum of
 * the best 3 out of 4 rolls of a 6-sided die.  Each class then decides
 * which priority will be given for the best to worst stats.
 */
void roll_real_abils(struct char_data * ch)
{
  int i, j, k, temp;
  ubyte table[6];
  ubyte rolls[4];

  for (i = 0; i < 6; i++)
    table[i] = 0;

  for (i = 0; i < 6; i++) {

    for (j = 0; j < 4; j++)
      rolls[j] = number(1, 6);

    temp = rolls[0] + rolls[1] + rolls[2] + rolls[3] -
      MIN(rolls[0], MIN(rolls[1], MIN(rolls[2], rolls[3])));

    for (k = 0; k < 6; k++)
      if (table[k] < temp) {
	temp ^= table[k];
	table[k] ^= temp;
	temp ^= table[k];
      }
  }

  ch->real_abils.str_add = 0;

  switch (GET_CLASS(ch)) {
  case CLASS_MAGIC_USER:
    ch->real_abils.intel = table[0];
    ch->real_abils.wis = table[1];
    ch->real_abils.dex = table[2];
    ch->real_abils.str = table[3];
    ch->real_abils.con = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_CLERIC:
    ch->real_abils.wis = table[0];
    ch->real_abils.intel = table[1];
    ch->real_abils.str = table[2];
    ch->real_abils.dex = table[3];
    ch->real_abils.con = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_THIEF:
    ch->real_abils.dex = table[0];
    ch->real_abils.str = table[1];
    ch->real_abils.con = table[2];
    ch->real_abils.intel = table[3];
    ch->real_abils.wis = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_WARRIOR:
    ch->real_abils.str = table[0];
    ch->real_abils.dex = table[1];
    ch->real_abils.con = table[2];
    ch->real_abils.wis = table[3];
    ch->real_abils.intel = table[4];
    ch->real_abils.cha = table[5];
    if (ch->real_abils.str == 18)
      ch->real_abils.str_add = number(0, 100);
    break;
  }
  ch->aff_abils = ch->real_abils;
}


/* Some initializations for characters, including initial skills */
void do_start(struct char_data * ch)
{
  GET_LEVEL(ch) = 1;
  GET_EXP(ch) = 1;

  set_title(ch, NULL);
  roll_real_abils(ch);
  ch->points.max_hit = 10;

  switch (GET_CLASS(ch)) {

  case CLASS_MAGIC_USER:
    break;

  case CLASS_CLERIC:
    break;

  case CLASS_THIEF:
    SET_SKILL(ch, SKILL_SNEAK, 10);
    SET_SKILL(ch, SKILL_HIDE, 5);
    SET_SKILL(ch, SKILL_STEAL, 15);
    SET_SKILL(ch, SKILL_BACKSTAB, 10);
    SET_SKILL(ch, SKILL_PICK_LOCK, 10);
    SET_SKILL(ch, SKILL_TRACK, 10);
    break;

  case CLASS_WARRIOR:
    break;
  }

  advance_level(ch);
  sprintf(buf, "%s advanced to level %d", GET_NAME(ch), GET_LEVEL(ch));
  mudlog(buf, BRF, MAX(LVL_IMMORT, GET_INVIS_LEV(ch)), TRUE);

  GET_HIT(ch) = GET_MAX_HIT(ch);
  GET_MANA(ch) = GET_MAX_MANA(ch);
  GET_MOVE(ch) = GET_MAX_MOVE(ch);

  GET_COND(ch, THIRST) = 24;
  GET_COND(ch, FULL) = 24;
  GET_COND(ch, DRUNK) = 0;

  ch->player.time.played = 0;
  ch->player.time.logon = time(0);

  if (siteok_everyone)
    SET_BIT(PLR_FLAGS(ch), PLR_SITEOK);
}



/*
 * This function controls the change to maxmove, maxmana, and maxhp for
 * each class every time they gain a level.
 */
void advance_level(struct char_data * ch)
{
  int add_hp, add_mana = 0, add_move = 0, i;

  add_hp = con_app[GET_CON(ch)].hitp;

  switch (GET_CLASS(ch)) {

  case CLASS_MAGIC_USER:
    add_hp += number(3, 8);
    add_mana = number(GET_LEVEL(ch), (int) (1.5 * GET_LEVEL(ch)));
    add_mana = MIN(add_mana, 10);
    add_move = number(0, 2);
    break;

  case CLASS_CLERIC:
    add_hp += number(5, 10);
    add_mana = number(GET_LEVEL(ch), (int) (1.5 * GET_LEVEL(ch)));
    add_mana = MIN(add_mana, 10);
    add_move = number(0, 2);
    break;

  case CLASS_THIEF:
    add_hp += number(7, 13);
    add_mana = 0;
    add_move = number(1, 3);
    break;

  case CLASS_WARRIOR:
    add_hp += number(10, 15);
    add_mana = 0;
    add_move = number(1, 3);
    break;
  }

  ch->points.max_hit += MAX(1, add_hp);
  ch->points.max_move += MAX(1, add_move);

  if (GET_LEVEL(ch) > 1)
    ch->points.max_mana += add_mana;

  if (IS_MAGIC_USER(ch) || IS_CLERIC(ch))
    GET_PRACTICES(ch) += MAX(2, wis_app[GET_WIS(ch)].bonus);
  else
    GET_PRACTICES(ch) += MIN(2, MAX(1, wis_app[GET_WIS(ch)].bonus));

  if (GET_LEVEL(ch) >= LVL_IMMORT) {
    for (i = 0; i < 3; i++)
      GET_COND(ch, i) = (char) -1;
    SET_BIT(PRF_FLAGS(ch), PRF_HOLYLIGHT);
  }

  save_char(ch, NOWHERE);
}


/*
 * This simply calculates the backstab multiplier based on a character's
 * level.  This used to be an array, but was changed to be a function so
 * that it would be easier to add more levels to your MUD.  This doesn't
 * really create a big performance hit because it's not used very often.
 */
int backstab_mult(int level)
{
  if (level <= 0)
    return 1;	  /* level 0 */
  else if (level <= 7)
    return 2;	  /* level 1 - 7 */
  else if (level <= 13)
    return 3;	  /* level 8 - 13 */
  else if (level <= 20)
    return 4;	  /* level 14 - 20 */
  else if (level <= 28)
    return 5;	  /* level 21 - 28 */
  else if (level < LVL_IMMORT)
    return 6;	  /* all remaining mortal levels */
  else
    return 20;	  /* immortals */
}


/*
 * invalid_class is used by handler.c to determine if a piece of equipment is
 * usable by a particular class, based on the ITEM_ANTI_{class} bitvectors.
 */

int invalid_class(struct char_data *ch, struct obj_data *obj) {
  if ((IS_OBJ_STAT(obj, ITEM_ANTI_MAGIC_USER) && IS_MAGIC_USER(ch)) ||
      (IS_OBJ_STAT(obj, ITEM_ANTI_CLERIC) && IS_CLERIC(ch)) ||
      (IS_OBJ_STAT(obj, ITEM_ANTI_WARRIOR) && IS_WARRIOR(ch)) ||
      (IS_OBJ_STAT(obj, ITEM_ANTI_THIEF) && IS_THIEF(ch)))
	return 1;
  else
	return 0;
}




/*
 * SPELLS AND SKILLS.  This area defines which spells are assigned to
 * which classes, and the minimum level the character must be to use
 * the spell or skill.
 */
void init_spell_levels(void)
{
  /* MAGES */
  spell_level(SPELL_MAGIC_MISSILE, CLASS_MAGIC_USER, 1);
  spell_level(SPELL_DETECT_INVIS, CLASS_MAGIC_USER, 2);
  spell_level(SPELL_DETECT_MAGIC, CLASS_MAGIC_USER, 2);
  spell_level(SPELL_CHILL_TOUCH, CLASS_MAGIC_USER, 3);
  spell_level(SPELL_INFRAVISION, CLASS_MAGIC_USER, 3);
  spell_level(SPELL_INVISIBLE, CLASS_MAGIC_USER, 4);
  spell_level(SPELL_ARMOR, CLASS_MAGIC_USER, 4);
  spell_level(SPELL_BURNING_HANDS, CLASS_MAGIC_USER, 5);
  spell_level(SPELL_LOCATE_OBJECT, CLASS_MAGIC_USER, 6);
  spell_level(SPELL_STRENGTH, CLASS_MAGIC_USER, 6);
  spell_level(SPELL_SHOCKING_GRASP, CLASS_MAGIC_USER, 7);
  spell_level(SPELL_SLEEP, CLASS_MAGIC_USER, 8);
  spell_level(SPELL_LIGHTNING_BOLT, CLASS_MAGIC_USER, 9);
  spell_level(SPELL_BLINDNESS, CLASS_MAGIC_USER, 9);
  spell_level(SPELL_DETECT_POISON, CLASS_MAGIC_USER, 10);
  spell_level(SPELL_COLOR_SPRAY, CLASS_MAGIC_USER, 11);
  spell_level(SPELL_ENERGY_DRAIN, CLASS_MAGIC_USER, 13);
  spell_level(SPELL_CURSE, CLASS_MAGIC_USER, 14);
  spell_level(SPELL_POISON, CLASS_MAGIC_USER, 14);
  spell_level(SPELL_FIREBALL, CLASS_MAGIC_USER, 15);
  spell_level(SPELL_CHARM, CLASS_MAGIC_USER, 16);
  spell_level(SPELL_ENCHANT_WEAPON, CLASS_MAGIC_USER, 26);
  spell_level(SPELL_CLONE, CLASS_MAGIC_USER, 30);

  /* CLERICS */
  spell_level(SPELL_CURE_LIGHT, CLASS_CLERIC, 1);
  spell_level(SPELL_ARMOR, CLASS_CLERIC, 1);
  spell_level(SPELL_CREATE_FOOD, CLASS_CLERIC, 2);
  spell_level(SPELL_CREATE_WATER, CLASS_CLERIC, 2);
  spell_level(SPELL_DETECT_POISON, CLASS_CLERIC, 3);
  spell_level(SPELL_DETECT_ALIGN, CLASS_CLERIC, 4);
  spell_level(SPELL_CURE_BLIND, CLASS_CLERIC, 4);
  spell_level(SPELL_BLESS, CLASS_CLERIC, 5);
  spell_level(SPELL_DETECT_INVIS, CLASS_CLERIC, 6);
  spell_level(SPELL_BLINDNESS, CLASS_CLERIC, 6);
  spell_level(SPELL_INFRAVISION, CLASS_CLERIC, 7);
  spell_level(SPELL_PROT_FROM_EVIL, CLASS_CLERIC, 8);
  spell_level(SPELL_POISON, CLASS_CLERIC, 8);
  spell_level(SPELL_GROUP_ARMOR, CLASS_CLERIC, 9);
  spell_level(SPELL_CURE_CRITIC, CLASS_CLERIC, 9);
  spell_level(SPELL_SUMMON, CLASS_CLERIC, 10);
  spell_level(SPELL_REMOVE_POISON, CLASS_CLERIC, 10);
  spell_level(SPELL_WORD_OF_RECALL, CLASS_CLERIC, 12);
  spell_level(SPELL_EARTHQUAKE, CLASS_CLERIC, 12);
  spell_level(SPELL_DISPEL_EVIL, CLASS_CLERIC, 14);
  spell_level(SPELL_DISPEL_GOOD, CLASS_CLERIC, 14);
  spell_level(SPELL_SANCTUARY, CLASS_CLERIC, 15);
  spell_level(SPELL_CALL_LIGHTNING, CLASS_CLERIC, 15);
  spell_level(SPELL_HEAL, CLASS_CLERIC, 16);
  spell_level(SPELL_CONTROL_WEATHER, CLASS_CLERIC, 17);
  spell_level(SPELL_SENSE_LIFE, CLASS_CLERIC, 18);
  spell_level(SPELL_HARM, CLASS_CLERIC, 19);
  spell_level(SPELL_GROUP_HEAL, CLASS_CLERIC, 22);
  spell_level(SPELL_REMOVE_CURSE, CLASS_CLERIC, 26);

  /* THIEVES */
  spell_level(SKILL_SNEAK, CLASS_THIEF, 1);
  spell_level(SKILL_PICK_LOCK, CLASS_THIEF, 2);
  spell_level(SKILL_BACKSTAB, CLASS_THIEF, 3);
  spell_level(SKILL_STEAL, CLASS_THIEF, 4);
  spell_level(SKILL_HIDE, CLASS_THIEF, 5);
  spell_level(SKILL_TRACK, CLASS_THIEF, 6);

  /* WARRIORS */
  spell_level(SKILL_KICK, CLASS_WARRIOR, 1);
  spell_level(SKILL_RESCUE, CLASS_WARRIOR, 3);
  spell_level(SKILL_TRACK, CLASS_WARRIOR, 9);
  spell_level(SKILL_BASH, CLASS_WARRIOR, 12);
}


/*
 * This is the exp given to implementors -- it must always be greater
 * than the exp required for immortality, plus at least 20,000 or so.
 */
#define EXP_MAX  77500000

/* Function to return the exp required for each class/level */
int level_exp(int chclass, int level)
{
  if (level > LVL_IMPL || level < 0) {
    log("SYSERR: Requesting exp for invalid level %d!", level);
    return 0;
  }

  /*
   * Gods have exp close to EXP_MAX.  This statement should never have to
   * changed, regardless of how many mortal or immortal levels exist.
   */
   if (level > LVL_IMMORT) {
     return EXP_MAX - ((LVL_IMPL - level) * 1000);
   }

  /* Exp required for normal mortals is below */

  switch (chclass) {

    case CLASS_MAGIC_USER:
    switch (level) {
      case  0: return 0;
      case  1: return 1;
      case  2: return 2000;
      case  3: return 4000;
      case  4: return 8000;
      case  5: return 16000;
      case  6: return 32000;
      case  7: return 64000;
      case  8: return 125000;
      case  9: return 250000;
      case 10: return 500000;
      case 11: return 750000;
      case 12: return 1000000;
      case 13: return 1250000;
      case 14: return 1500000;
      case 15: return 1850000;
      case 16: return 2200000;
      case 17: return 2550000; 
      case 18: return 2900000; 
      case 19: return 3250000; 
      case 20: return 3600000; 
      case 21: return 3900000; 
      case 22: return 4200000;
      case 23: return 4500000;
      case 24: return 4800000;
      case 25: return 5150000;
      case 26: return 5500000;
      case 27: return 5950000;
      case 28: return 6400000;
      case 29: return 6850000;
      case 30: return 7400000;
      case 31: return 8400000;
      case 32: return 9400000;
      case 33: return 10400000;
      case 34: return 11400000;
      case 35: return 12400000;
      case 36: return 13400000;
      case 37: return 14400000;
      case 38: return 15400000;
      case 39: return 16400000;
      case 40: return 17400000;
      case 41: return 18400000;
      case 42: return 19400000;
      case 43: return 20400000;
      case 44: return 21400000;
      case 45: return 21900000;
      case 46: return 22400000;
      case 47: return 23400000;
      case 48: return 24400000;
      case 49: return 25400000;
      case 50: return 26400000;
      case 51: return 27400000;
      case 52: return 28400000;
      case 53: return 29400000;
      case 54: return 30400000;
      case 55: return 31400000;
      case 56: return 32400000;
      case 57: return 33400000;
      case 58: return 34400000;
      case 59: return 35400000;
      case 60: return 36400000;
      case 61: return 37400000;
      case 62: return 38400000;
      case 63: return 39400000;
      case 64: return 40400000;
      case 65: return 41400000;
      case 66: return 42400000;
      case 67: return 43400000;
      case 68: return 44400000;
      case 69: return 45400000;
      case 70: return 46400000;
      case 71: return 47400000;
      case 72: return 48400000;
      case 73: return 49400000;
      case 74: return 50400000;
      case 75: return 51400000;
      case 76: return 52400000;
      case 77: return 53400000;
      case 78: return 54400000;
      case 79: return 55400000;
      case 80: return 56400000;
      case 81: return 57400000;
      case 82: return 58400000;
      case 83: return 59400000;
      case 84: return 60400000;
      case 85: return 61400000;
      case 86: return 62400000;
      case 87: return 63400000;
      case 88: return 64400000;
      case 89: return 65400000;
      case 90: return 66400000;
      case 91: return 67400000;
      case 92: return 68400000;
      case 93: return 69400000;
      case 94: return 70400000;
      case 95: return 71400000;

      /* add new levels here */
      case LVL_IMMORT: return 75500000;
    }
    break;

    case CLASS_CLERIC:
    switch (level) {
      case  0: return 0;
      case  1: return 1;
      case  2: return 2000;
      case  3: return 4000;
      case  4: return 8000;
      case  5: return 16000;
      case  6: return 32000;
      case  7: return 64000;
      case  8: return 125000;
      case  9: return 250000;
      case 10: return 500000;
      case 11: return 750000;
      case 12: return 1000000;
      case 13: return 1250000;
      case 14: return 1500000;
      case 15: return 1850000;
      case 16: return 2200000;
      case 17: return 2550000; 
      case 18: return 2900000; 
      case 19: return 3250000; 
      case 20: return 3600000; 
      case 21: return 3900000; 
      case 22: return 4200000;
      case 23: return 4500000;
      case 24: return 4800000;
      case 25: return 5150000;
      case 26: return 5500000;
      case 27: return 5950000;
      case 28: return 6400000;
      case 29: return 6850000;
      case 30: return 7400000;
      case 31: return 8400000;
      case 32: return 9400000;
      case 33: return 10400000;
      case 34: return 11400000;
      case 35: return 12400000;
      case 36: return 13400000;
      case 37: return 14400000;
      case 38: return 15400000;
      case 39: return 16400000;
      case 40: return 17400000;
      case 41: return 18400000;
      case 42: return 19400000;
      case 43: return 20400000;
      case 44: return 21400000;
      case 45: return 21900000;
      case 46: return 22400000;
      case 47: return 23400000;
      case 48: return 24400000;
      case 49: return 25400000;
      case 50: return 26400000;
      case 51: return 27400000;
      case 52: return 28400000;
      case 53: return 29400000;
      case 54: return 30400000;
      case 55: return 31400000;
      case 56: return 32400000;
      case 57: return 33400000;
      case 58: return 34400000;
      case 59: return 35400000;
      case 60: return 36400000;
      case 61: return 37400000;
      case 62: return 38400000;
      case 63: return 39400000;
      case 64: return 40400000;
      case 65: return 41400000;
      case 66: return 42400000;
      case 67: return 43400000;
      case 68: return 44400000;
      case 69: return 45400000;
      case 70: return 46400000;
      case 71: return 47400000;
      case 72: return 48400000;
      case 73: return 49400000;
      case 74: return 50400000;
      case 75: return 51400000;
      case 76: return 52400000;
      case 77: return 53400000;
      case 78: return 54400000;
      case 79: return 55400000;
      case 80: return 56400000;
      case 81: return 57400000;
      case 82: return 58400000;
      case 83: return 59400000;
      case 84: return 60400000;
      case 85: return 61400000;
      case 86: return 62400000;
      case 87: return 63400000;
      case 88: return 64400000;
      case 89: return 65400000;
      case 90: return 66400000;
      case 91: return 67400000;
      case 92: return 68400000;
      case 93: return 69400000;
      case 94: return 70400000;
      case 95: return 71400000;

      /* add new levels here */
      case LVL_IMMORT: return 75500000;
    }
    break;

    case CLASS_THIEF:
    switch (level) {
      case  0: return 0;
      case  1: return 1;
      case  2: return 2000;
      case  3: return 4000;
      case  4: return 8000;
      case  5: return 16000;
      case  6: return 32000;
      case  7: return 64000;
      case  8: return 125000;
      case  9: return 250000;
      case 10: return 500000;
      case 11: return 750000;
      case 12: return 1000000;
      case 13: return 1250000;
      case 14: return 1500000;
      case 15: return 1850000;
      case 16: return 2200000;
      case 17: return 2550000; 
      case 18: return 2900000; 
      case 19: return 3250000; 
      case 20: return 3600000; 
      case 21: return 3900000; 
      case 22: return 4200000;
      case 23: return 4500000;
      case 24: return 4800000;
      case 25: return 5150000;
      case 26: return 5500000;
      case 27: return 5950000;
      case 28: return 6400000;
      case 29: return 6850000;
      case 30: return 7400000;
      case 31: return 8400000;
      case 32: return 9400000;
      case 33: return 10400000;
      case 34: return 11400000;
      case 35: return 12400000;
      case 36: return 13400000;
      case 37: return 14400000;
      case 38: return 15400000;
      case 39: return 16400000;
      case 40: return 17400000;
      case 41: return 18400000;
      case 42: return 19400000;
      case 43: return 20400000;
      case 44: return 21400000;
      case 45: return 21900000;
      case 46: return 22400000;
      case 47: return 23400000;
      case 48: return 24400000;
      case 49: return 25400000;
      case 50: return 26400000;
      case 51: return 27400000;
      case 52: return 28400000;
      case 53: return 29400000;
      case 54: return 30400000;
      case 55: return 31400000;
      case 56: return 32400000;
      case 57: return 33400000;
      case 58: return 34400000;
      case 59: return 35400000;
      case 60: return 36400000;
      case 61: return 37400000;
      case 62: return 38400000;
      case 63: return 39400000;
      case 64: return 40400000;
      case 65: return 41400000;
      case 66: return 42400000;
      case 67: return 43400000;
      case 68: return 44400000;
      case 69: return 45400000;
      case 70: return 46400000;
      case 71: return 47400000;
      case 72: return 48400000;
      case 73: return 49400000;
      case 74: return 50400000;
      case 75: return 51400000;
      case 76: return 52400000;
      case 77: return 53400000;
      case 78: return 54400000;
      case 79: return 55400000;
      case 80: return 56400000;
      case 81: return 57400000;
      case 82: return 58400000;
      case 83: return 59400000;
      case 84: return 60400000;
      case 85: return 61400000;
      case 86: return 62400000;
      case 87: return 63400000;
      case 88: return 64400000;
      case 89: return 65400000;
      case 90: return 66400000;
      case 91: return 67400000;
      case 92: return 68400000;
      case 93: return 69400000;
      case 94: return 70400000;
      case 95: return 71400000;
      /* add new levels here */
      case LVL_IMMORT: return 75500000;
    }
    break;

    case CLASS_WARRIOR:
    switch (level) {
      case  0: return 0;
      case  1: return 1;
      case  2: return 2000;
      case  3: return 4000;
      case  4: return 8000;
      case  5: return 16000;
      case  6: return 32000;
      case  7: return 64000;
      case  8: return 125000;
      case  9: return 250000;
      case 10: return 500000;
      case 11: return 750000;
      case 12: return 1000000;
      case 13: return 1250000;
      case 14: return 1500000;
      case 15: return 1850000;
      case 16: return 2200000;
      case 17: return 2550000; 
      case 18: return 2900000; 
      case 19: return 3250000; 
      case 20: return 3600000; 
      case 21: return 3900000; 
      case 22: return 4200000;
      case 23: return 4500000;
      case 24: return 4800000;
      case 25: return 5150000;
      case 26: return 5500000;
      case 27: return 5950000;
      case 28: return 6400000;
      case 29: return 6850000;
      case 30: return 7400000;
      case 31: return 8400000;
      case 32: return 9400000;
      case 33: return 10400000;
      case 34: return 11400000;
      case 35: return 12400000;
      case 36: return 13400000;
      case 37: return 14400000;
      case 38: return 15400000;
      case 39: return 16400000;
      case 40: return 17400000;
      case 41: return 18400000;
      case 42: return 19400000;
      case 43: return 20400000;
      case 44: return 21400000;
      case 45: return 21900000;
      case 46: return 22400000;
      case 47: return 23400000;
      case 48: return 24400000;
      case 49: return 25400000;
      case 50: return 26400000;
      case 51: return 27400000;
      case 52: return 28400000;
      case 53: return 29400000;
      case 54: return 30400000;
      case 55: return 31400000;
      case 56: return 32400000;
      case 57: return 33400000;
      case 58: return 34400000;
      case 59: return 35400000;
      case 60: return 36400000;
      case 61: return 37400000;
      case 62: return 38400000;
      case 63: return 39400000;
      case 64: return 40400000;
      case 65: return 41400000;
      case 66: return 42400000;
      case 67: return 43400000;
      case 68: return 44400000;
      case 69: return 45400000;
      case 70: return 46400000;
      case 71: return 47400000;
      case 72: return 48400000;
      case 73: return 49400000;
      case 74: return 50400000;
      case 75: return 51400000;
      case 76: return 52400000;
      case 77: return 53400000;
      case 78: return 54400000;
      case 79: return 55400000;
      case 80: return 56400000;
      case 81: return 57400000;
      case 82: return 58400000;
      case 83: return 59400000;
      case 84: return 60400000;
      case 85: return 61400000;
      case 86: return 62400000;
      case 87: return 63400000;
      case 88: return 64400000;
      case 89: return 65400000;
      case 90: return 66400000;
      case 91: return 67400000;
      case 92: return 68400000;
      case 93: return 69400000;
      case 94: return 70400000;
      case 95: return 71400000;
      /* add new levels here */
      case LVL_IMMORT: return 75500000;
    }
    break;
  }

  /*
   * This statement should never be reached if the exp tables in this function
   * are set up properly.  If you see exp of 123456 then the tables above are
   * incomplete -- so, complete them!
   */
  log("SYSERR: XP tables not set up correctly in class.c!");
  return 123456;
}


/* 
 * Default titles of male characters.
 */
const char *title_male(int chclass, int level)
{
  if (level <= 0 || level > LVL_IMPL)
    return "the Man";
  if (level == LVL_IMPL)
    return "the Implementor";

  switch (chclass) {

    case CLASS_MAGIC_USER:
    switch (level) {
      case  1: return "the Apprentice of Magic";
      case  2: return "the Apprentice of Magic";
      case  3: return "the Apprentice of Magic";
      case  4: return "the Apprentice of Magic";
      case  5: return "the Spell Student";
      case  6: return "the Spell Student";
      case  7: return "the Spell Student";
      case  8: return "the Spell Student";
      case  9: return "the Spell Student";
      case  10: return "the Scholar of Magic";
      case  11: return "the Scholar of Magic";
      case  12: return "the Scholar of Magic";
      case  13: return "the Scholar of Magic";
      case  14: return "the Scholar of Magic";
      case  15: return "the Delver in Spells";
      case  16: return "the Delver in Spells";
      case  17: return "the Delver in Spells";
      case  18: return "the Delver in Spells";
      case  19: return "the Medium of Magic";
      case  20: return "the Medium of Magic";
      case  21: return "the Medium of Magic";
      case  22: return "the Medium of Magic";
      case  23: return "the Medium of Magic";
      case  24: return "the Scribe of Magic";
      case  25: return "the Scribe of Magic";
      case  26: return "the Scribe of Magic";
      case  27: return "the Scribe of Magic";
      case  28: return "the Scribe of Magic";
      case  29: return "the Seer";
      case  30: return "the Seer";
      case  31: return "the Seer";
      case  32: return "the Seer";
      case  33: return "the Sage";
      case  34: return "the Sage";
      case  35: return "the Sage";
      case  36: return "the Sage";
      case  37: return "the Sage";
      case  38: return "the Illusionist";
      case  39: return "the Illusionist";
      case  40: return "the Illusionist";
      case  41: return "the Illusionist";
      case  42: return "the Illusionist";
      case 43: return "the Abjurer";
      case 44: return "the Abjurer";
      case 45: return "the Abjurer";
      case 46: return "the Abjurer";
      case 47: return "the Abjurer";
      case 48: return "the Invoker";
      case 49: return "the Invoker";
      case 50: return "the Invoker";
      case 51: return "the Invoker";
      case 52: return "the Invoker";
      case 53: return "the Enchanter";
      case 54: return "the Enchanter";
      case 55: return "the Enchanter";
      case 56: return "the Enchanter";
      case 57: return "the Enchanter";
      case 58: return "the Conjurer";
      case 59: return "the Conjurer";
      case 60: return "the Conjurer";
      case 61: return "the Conjurer";
      case 62: return "the Magician";
      case 63: return "the Magician";
      case 64: return "the Magician";
      case 65: return "the Magician";
      case 66: return "the Creator";
      case 67: return "the Creator";
      case 68: return "the Creator";
      case 69: return "the Creator";
      case 70: return "the Savant";
      case 71: return "the Savant";
      case 72: return "the Savant";
      case 73: return "the Savant";
      case 74: return "the Magus";
      case 75: return "the Magus";
      case 76: return "the Magus";
      case 77: return "the Magus";
      case 78: return "the Wizard";
      case 79: return "the Wizard";
      case 80: return "the Wizard";
      case 81: return "the Warlock";
      case 82: return "the Warlock";
      case 83: return "the Sorcerer";
      case 84: return "the Sorcerer";
      case 85: return "the Sorcerer";
      case 86: return "the Necromancer";
      case 87: return "the Thaumaturge";
      case 88: return "the Student of the Occult";
      case 89: return "the Disciple of the Uncanny";
      case 90: return "the Minor Elemental";
      case 91: return "the Greater Elemental";
      case 92: return "the Crafter of Magics";
      case 93: return "the Shaman";
      case 94: return "the Keeper of Talismans";
      case 95: return "the Archmage";
      case LVL_IMMORT: return "the Immortal Warlock";
      case LVL_GOD: return "the Avatar of Magic";
      case LVL_GRGOD: return "the God of Magic";
      default: return "the Mage";
    }
    break;

    case CLASS_CLERIC:
    switch (level) {
      case  1: return "the Believer";
      case  2: return "the Believer";
      case  3: return "the Believer";
      case  4: return "the Believer";
      case  5: return "the Believer";
      case  6: return "the Believer";
      case  7: return "the Attendant";
      case  8: return "the Attendant";
      case  9: return "the Attendant";
      case  10: return "the Attendant";
      case  11: return "the Attendant";
      case  12: return "the Attendant";
      case  13: return "the Attendant";
      case  14: return "the Acolyte";
      case  15: return "the Acolyte";
      case  16: return "the Acolyte";
      case  17: return "the Acolyte";
      case  18: return "the Acolyte";
      case  19: return "the Acolyte";
      case  20: return "the Acolyte";
      case  21: return "the Novice";
      case  22: return "the Novice";
      case  23: return "the Novice";
      case  24: return "the Novice";
      case  25: return "the Novice";
      case  26: return "the Novice";
      case  27: return "the Missionary";
      case  28: return "the Missionary";
      case  29: return "the Missionary";
      case  30: return "the Missionary";
      case  31: return "the Missionary";
      case  32: return "the Missionary";
      case  33: return "the Adept";
      case  34: return "the Adept";
      case  35: return "the Adept";
      case  36: return "the Adept";
      case  37: return "the Adept";
      case  38: return "the Adept";
      case  39: return "the Adept";
      case  40: return "the Adept";
      case  41: return "the Adept";
      case  42: return "the Adept";
      case  43: return "the Deacon";
      case  44: return "the Deacon";
      case  45: return "the Deacon";
      case  46: return "the Deacon";
      case  47: return "the Deacon";
      case  48: return "the Deacon";
      case  49: return "the Deacon";
      case  50: return "the Deacon";
      case  51: return "the Deacon";
      case  52: return "the Deacon";
      case  53: return "the Vicar";
      case  54: return "the Vicar";
      case  55: return "the Vicar";
      case  56: return "the Vicar";
      case  57: return "the Priest";
      case  58: return "the Priest";
      case  59: return "the Priest";
      case  60: return "the Priest";
      case 61: return "the Minister";
      case 62: return "the Minister";
      case 63: return "the Minister";
      case 64: return "the Minister";
      case 65: return "the Canon";
      case 66: return "the Canon";
      case 67: return "the Canon";
      case 68: return "the Canon";
      case 69: return "the Levite";
      case 70: return "the Levite";
      case 71: return "the Levite";
      case 72: return "the Levite";
      case 73: return "the Curate";
      case 74: return "the Curate";
      case 75: return "the Curate";
      case 76: return "the Curate";
      case 77: return "the Monk";
      case 78: return "the Monk";
      case 79: return "the Monk";
      case 80: return "the Monk";
      case 81: return "the Healer";
      case 82: return "the Healer";
      case 83: return "the Healer";
      case 84: return "the Healer";
      case 85: return "the Healer";
      case 86: return "the Chaplain";
      case 87: return "the Chaplain";
      case 88: return "the Chaplain";
      case 89: return "the Chaplain";
      case 90: return "the Expositor";
      case 91: return "the Bishop";
      case 92: return "the Bishop";
      case 93: return "the Arch Bishop";
      case 94: return "the Arch Bishop";
      case 95: return "the Patriarch";
      /* no one ever thought up these titles 21-30 */
      case LVL_IMMORT: return "the Immortal Cardinal";
      case LVL_GOD: return "the Inquisitor";
      case LVL_GRGOD: return "the God of good and evil";
      default: return "the Cleric";
    }
    break;

    case CLASS_THIEF:
    switch (level) {
      case  1: return "the Pilferer";
      case  2: return "the Pilferer";
      case  3: return "the Footpad";
      case  4: return "the Footpad";
      case  5: return "the Filcher";
      case  6: return "the Filcher";
      case  7: return "the Pick-Pocket";
      case  8: return "the Pick-Pocket";
      case  9: return "the Pick-Pocket";
      case  10: return "the Sneak";
      case  11: return "the Sneak";
      case  12: return "the Sneak";
      case  13: return "the Sneak";
      case  14: return "the Sneak";
      case  15: return "the Pincher";
      case  16: return "the Pincher";
      case  17: return "the Pincher";
      case  18: return "the Pincher";
      case  19: return "the Pincher";
      case  20: return "the Pincher";
      case  21: return "the Pincher";
      case  22: return "the Pincher";
      case  23: return "the Cut-Purse";
      case  24: return "the Cut-Purse";
      case  25: return "the Cut-Purse";
      case  26: return "the Cut-Purse";
      case  27: return "the Cut-Purse";
      case  28: return "the Cut-Purse";
      case  29: return "the Cut-Purse";
      case  30: return "the Cut-Purse";
      case  31: return "the Cut-Purse";
      case  32: return "the Cut-Purse";
      case  33: return "the Snatcher";
      case  34: return "the Snatcher";
      case  35: return "the Snatcher";
      case  36: return "the Snatcher";
      case  37: return "the Snatcher";
      case  38: return "the Snatcher";
      case  39: return "the Snatcher";
      case  40: return "the Snatcher";
      case  41: return "the Snatcher";
      case  42: return "the Snatcher";
      case  43: return "the Sharper";
      case  44: return "the Sharper";
      case  45: return "the Sharper";
      case  46: return "the Sharper";
      case  47: return "the Sharper";
      case  48: return "the Sharper";
      case  49: return "the Sharper";
      case  50: return "the Sharper";
      case  51: return "the Sharper";
      case  52: return "the Sharper";
      case 53: return "the Rogue";
      case 54: return "the Rogue";
      case 55: return "the Rogue";
      case 56: return "the Rogue";
      case 57: return "the Rogue";
      case 58: return "the Robber";
      case 59: return "the Robber";
      case 60: return "the Robber";
      case 61: return "the Robber";
      case 62: return "the Robber";
      case 63: return "the Magsman";
      case 64: return "the Magsman";
      case 65: return "the Magsman";
      case 66: return "the Magsman";
      case 67: return "the Magsman";
      case 68: return "the Highwayman";
      case 69: return "the Highwayman";
      case 70: return "the Highwayman";
      case 71: return "the Highwayman";
      case 72: return "the Highwayman";
      case 73: return "the Burglar";
      case 74: return "the Burglar";
      case 75: return "the Burglar";
      case 76: return "the Burglar";
      case 77: return "the Burglar";
      case 78: return "the Thief";
      case 79: return "the Thief";
      case 80: return "the Thief";
      case 81: return "the Thief";
      case 82: return "the Thief";
      case 83: return "the Knifer";
      case 84: return "the Knifer";
      case 85: return "the Knifer";
      case 86: return "the Knifer";
      case 87: return "the Knifer";
      case 88: return "the Quick-Blade";
      case 89: return "the Quick-Blade";
      case 90: return "the Quick-Blade";
      case 91: return "the Killer";
      case 92: return "the Killer";
      case 93: return "the Brigand";
      case 94: return "the Brigand";
      case 95: return "the Cut-Throat";
      /* no one ever thought up these titles 21-30 */
      case LVL_IMMORT: return "the Immortal Assasin";
      case LVL_GOD: return "the Demi God of thieves";
      case LVL_GRGOD: return "the God of thieves and tradesmen";
      default: return "the Thief";
    }
    break;

    case CLASS_WARRIOR:
    switch(level) {
      case  1: return "the Swordpupil";
      case  2: return "the Swordpupil";
      case  3: return "the Recruit";
      case  4: return "the Recruit";
      case  5: return "the Sentry";
      case  6: return "the Sentry";
      case  7: return "the Fighter";
      case  8: return "the Fighter";
      case  9: return "the Fighter";
      case  10: return "the Soldier";
      case  11: return "the Soldier";
      case  12: return "the Soldier";
      case  13: return "the Soldier";
      case  14: return "the Soldier";
      case  15: return "the Warrior";
      case  16: return "the Warrior";
      case  17: return "the Warrior";
      case  18: return "the Warrior";
      case  19: return "the Warrior";
      case  20: return "the Warrior";
      case  21: return "the Warrior";
      case  22: return "the Warrior";
      case  23: return "the Veteran";
      case  24: return "the Veteran";
      case  25: return "the Veteran";
      case  26: return "the Veteran";
      case  27: return "the Veteran";
      case  28: return "the Veteran";
      case  29: return "the Veteran";
      case  30: return "the Veteran";
      case  31: return "the Veteran";
      case  32: return "the Veteran";
      case  33: return "the Swordsman";
      case  34: return "the Swordsman";
      case  35: return "the Swordsman";
      case  36: return "the Swordsman";
      case  37: return "the Swordsman";
      case  38: return "the Swordsman";
      case  39: return "the Swordsman";
      case  40: return "the Swordsman";
      case  41: return "the Swordsman";
      case  42: return "the Swordsman";
      case  43: return "the Fencer";
      case  44: return "the Fencer";
      case  45: return "the Fencer";
      case  46: return "the Fencer";
      case  47: return "the Fencer";
      case  48: return "the Fencer";
      case  49: return "the Fencer";
      case  50: return "the Fencer";
      case  51: return "the Fencer";
      case  52: return "the Fencer";
      case 53: return "the Combatant";
      case 54: return "the Combatant";
      case 55: return "the Combatant";
      case 56: return "the Combatant";
      case 57: return "the Combatant";
      case 58: return "the Hero";
      case 59: return "the Hero";
      case 60: return "the Hero";
      case 61: return "the Hero";
      case 62: return "the Hero";
      case 63: return "the Myrmidon";
      case 64: return "the Myrmidon";
      case 65: return "the Myrmidon";
      case 66: return "the Myrmidon";
      case 67: return "the Myrmidon";
      case 68: return "the Swashbuckler";
      case 69: return "the Swashbuckler";
      case 70: return "the Swashbuckler";
      case 71: return "the Swashbuckler";
      case 72: return "the Swashbuckler";
      case 73: return "the Mercenary";
      case 74: return "the Mercenary";
      case 75: return "the Mercenary";
      case 76: return "the Mercenary";
      case 77: return "the Mercenary";
      case 78: return "the Swordmaster";
      case 79: return "the Swordmaster";
      case 80: return "the Swordmaster";
      case 81: return "the Swordmaster";
      case 82: return "the Swordmaster";
      case 83: return "the Lieutenant";
      case 84: return "the Lieutenant";
      case 85: return "the Lieutenant";
      case 86: return "the Lieutenant";
      case 87: return "the Lieutenant";
      case 88: return "the Champion";
      case 89: return "the Champion";
      case 90: return "the Champion";
      case 91: return "the Dragoon";
      case 92: return "the Dragoon";
      case 93: return "the Cavalier";
      case 94: return "the Cavalier";
      case 95: return "the Knight";
      /* no one ever thought up these titles 21-30 */
      case LVL_IMMORT: return "the Immortal Warlord";
      case LVL_GOD: return "the Extirpator";
      case LVL_GRGOD: return "the God of war";
      default: return "the Warrior";
    }
    break;
  }

  /* Default title for classes which do not have titles defined */
  return "the Classless";
}


/* 
 * Default titles of female characters.
 */
const char *title_female(int chclass, int level)
{
  if (level <= 0 || level > LVL_IMPL)
    return "the Woman";
  if (level == LVL_IMPL)
    return "the Implementress";

  switch (chclass) {

    case CLASS_MAGIC_USER:
    switch (level) {
      case  1: return "the Apprentice of Magic";
      case  2: return "the Apprentice of Magic";
      case  3: return "the Spell Student";
      case  4: return "the Spell Student";
      case  5: return "the Scholar of Magic";
      case  6: return "the Scholar of Magic";
      case  7: return "the Delveress in Spells";
      case  8: return "the Delveress in Spells";
      case  9: return "the Delveress in Spells";
      case 10: return "the Medium of Magic";
      case 11: return "the Medium of Magic";
      case 12: return "the Medium of Magic";
      case 13: return "the Medium of Magic";
      case 14: return "the Medium of Magic";
      case 15: return "the Scribess of Magic";
      case 16: return "the Scribess of Magic";
      case 17: return "the Scribess of Magic";
      case 18: return "the Scribess of Magic";
      case 19: return "the Scribess of Magic";
      case 20: return "the Scribess of Magic";
      case 21: return "the Scribess of Magic";
      case 22: return "the Scribess of Magic";
      case 23: return "the Seeress";
      case 24: return "the Seeress";
      case 25: return "the Seeress";
      case 26: return "the Seeress";
      case 27: return "the Seeress";
      case 28: return "the Seeress";
      case 29: return "the Seeress";
      case 30: return "the Seeress";
      case 31: return "the Seeress";
      case 32: return "the Seeress";
      case 33: return "the Sage";
      case 34: return "the Sage";
      case 35: return "the Sage";
      case 36: return "the Sage";
      case 37: return "the Sage";
      case 38: return "the Sage";
      case 39: return "the Sage";
      case 40: return "the Sage";
      case 41: return "the Sage";
      case 42: return "the Sage";
      case 43: return "the Illusionist";
      case 44: return "the Illusionist";
      case 45: return "the Illusionist";
      case 46: return "the Illusionist";
      case 47: return "the Illusionist";
      case 48: return "the Illusionist";
      case 49: return "the Illusionist";
      case 50: return "the Illusionist";
      case 51: return "the Illusionist";
      case 52: return "the Illusionist";
      case 53: return "the Abjuress";
      case 54: return "the Abjuress";
      case 55: return "the Abjuress";
      case 56: return "the Abjuress";
      case 57: return "the Abjuress";
      case 58: return "the Invoker";
      case 59: return "the Invoker";
      case 60: return "the Invoker";
      case 61: return "the Invoker";
      case 62: return "the Invoker";
      case 63: return "the Enchantress";
      case 64: return "the Enchantress";
      case 65: return "the Enchantress";
      case 66: return "the Enchantress";
      case 67: return "the Enchantress";
      case 68: return "the Conjuress";
      case 69: return "the Conjuress";
      case 70: return "the Conjuress";
      case 71: return "the Conjuress";
      case 72: return "the Conjuress";
      case 73: return "the Witch";
      case 74: return "the Witch";
      case 75: return "the Witch";
      case 76: return "the Witch";
      case 77: return "the Witch";
      case 78: return "the Creator";
      case 79: return "the Creator";
      case 80: return "the Creator";
      case 81: return "the Creator";
      case 82: return "the Creator";
      case 83: return "the Savant";
      case 84: return "the Craftess";
      case 85: return "the Wizard";
      case 86: return "the War Witch";
      case 87: return "the Sorceress";
      case 88: return "the Necromancress ";
      case 89: return "the Thaumaturgess ";
      case 90: return "the Disciple of the Uncanny ";
      case 91: return "the Minor Elementress ";
      case 92: return "the Greater Elementress ";
      case 93: return "the Crafter of Magics ";
      case 94: return "the Keeper of Talismans ";
      case 95: return "the Archwitch ";
      case LVL_IMMORT: return "the Immortal Enchantress";
      case LVL_GOD: return "the Empress of Magic";
      case LVL_GRGOD: return "the Goddess of Magic";
      default: return "the Witch";
    }
    break;

    case CLASS_CLERIC:
    switch (level) {
      case  1: return "the Believer";
      case  2: return "the Believer";
      case  3: return "the Attendant";
      case  4: return "the Attendant";
      case  5: return "the Acolyte";
      case  6: return "the Acolyte";
      case  7: return "the Novice";
      case  8: return "the Novice";
      case  9: return "the Novice";
      case 10: return "the Missionary";
      case 11: return "the Missionary";
      case 12: return "the Missionary";
      case 13: return "the Missionary";
      case 14: return "the Missionary";
      case 15: return "the Adept";
      case 16: return "the Adept";
      case 17: return "the Adept";
      case 18: return "the Adept";
      case 19: return "the Adept";
      case 20: return "the Adept";
      case 21: return "the Adept";
      case 22: return "the Adept";
      case 23: return "the Deaconess";
      case 24: return "the Deaconess";
      case 25: return "the Deaconess";
      case 26: return "the Deaconess";
      case 27: return "the Deaconess";
      case 28: return "the Deaconess";
      case 29: return "the Deaconess";
      case 30: return "the Deaconess";
      case 31: return "the Deaconess";
      case 32: return "the Deaconess";
      case 33: return "the Vicaress";
      case 34: return "the Vicaress";
      case 35: return "the Vicaress";
      case 36: return "the Vicaress";
      case 37: return "the Vicaress";
      case 38: return "the Vicaress";
      case 39: return "the Vicaress";
      case 40: return "the Vicaress";
      case 41: return "the Vicaress";
      case 42: return "the Vicaress";
      case 43: return "the Priestess";
      case 44: return "the Priestess";
      case 45: return "the Priestess";
      case 46: return "the Priestess";
      case 47: return "the Priestess";
      case 48: return "the Priestess";
      case 49: return "the Priestess";
      case 50: return "the Priestess";
      case 51: return "the Priestess";
      case 52: return "the Priestess";
      case 53: return "the Lady Minister";
      case 54: return "the Lady Minister";
      case 55: return "the Lady Minister";
      case 56: return "the Lady Minister";
      case 57: return "the Lady Minister";
      case 58: return "the Canon";
      case 59: return "the Canon";
      case 60: return "the Canon";
      case 61: return "the Canon";
      case 62: return "the Canon";
      case 63: return "the Levitess";
      case 64: return "the Levitess";
      case 65: return "the Levitess";
      case 66: return "the Levitess";
      case 67: return "the Levitess";
      case 68: return "the Curess";
      case 69: return "the Curess";
      case 70: return "the Curess";
      case 71: return "the Curess";
      case 72: return "the Curess";
      case 73: return "the Nunne";
      case 74: return "the Nunne";
      case 75: return "the Nunne";
      case 76: return "the Nunne";
      case 77: return "the Nunne";
      case 78: return "the Healess";
      case 79: return "the Healess";
      case 80: return "the Healess";
      case 81: return "the Healess";
      case 82: return "the Healess";
      case 83: return "the Chaplain";
      case 84: return "the Chaplain";
      case 85: return "the Chaplain";
      case 86: return "the Chaplain";
      case 87: return "the Chaplain";
      case 88: return "the Bishop";
      case 89: return "the Bishop";
      case 90: return "the Bishop";
      case 91: return "the Bishop";
      case 92: return "the Arch Lady of the Church";
      case 93: return "the Arch Lady of the Church";
      case 94: return "the Arch Lady of the Church";
      case 95: return "the Matriarch";
      /* no one ever thought up these titles 21-30 */
      case LVL_IMMORT: return "the Immortal Priestess";
      case LVL_GOD: return "the Inquisitress";
      case LVL_GRGOD: return "the Goddess of good and evil";
      default: return "the Cleric";
    }
    break;

    case CLASS_THIEF:
    switch (level) {
      case  1: return "the Pilferess";
      case  2: return "the Pilferess";
      case  3: return "the Footpad";
      case  4: return "the Footpad";
      case  5: return "the Filcheress";
      case  6: return "the Filcheress";
      case  7: return "the Pick-Pocket";
      case  8: return "the Pick-Pocket";
      case  9: return "the Pick-Pocket";
      case 10: return "the Sneak";
      case 11: return "the Sneak";
      case 12: return "the Sneak";
      case 13: return "the Sneak";
      case 14: return "the Sneak";
      case 15: return "the Pincheress";
      case 16: return "the Pincheress";
      case 17: return "the Pincheress";
      case 18: return "the Pincheress";
      case 19: return "the Pincheress";
      case 20: return "the Pincheress";
      case 21: return "the Pincheress";
      case 22: return "the Pincheress";
      case 23: return "the Cut-Purse";
      case 24: return "the Cut-Purse";
      case 25: return "the Cut-Purse";
      case 26: return "the Cut-Purse";
      case 27: return "the Cut-Purse";
      case 28: return "the Cut-Purse";
      case 29: return "the Cut-Purse";
      case 30: return "the Cut-Purse";
      case 31: return "the Cut-Purse";
      case 32: return "the Cut-Purse";
      case 33: return "the Snatcheress";
      case 34: return "the Snatcheress";
      case 35: return "the Snatcheress";
      case 36: return "the Snatcheress";
      case 37: return "the Snatcheress";
      case 38: return "the Snatcheress";
      case 39: return "the Snatcheress";
      case 40: return "the Snatcheress";
      case 41: return "the Snatcheress";
      case 42: return "the Snatcheress";
      case 43: return "the Sharpress";
      case 44: return "the Sharpress";
      case 45: return "the Sharpress";
      case 46: return "the Sharpress";
      case 47: return "the Sharpress";
      case 48: return "the Sharpress";
      case 49: return "the Sharpress";
      case 50: return "the Sharpress";
      case 51: return "the Sharpress";
      case 52: return "the Sharpress";
      case 53: return "the Rogue";
      case 54: return "the Rogue";
      case 55: return "the Rogue";
      case 56: return "the Rogue";
      case 57: return "the Rogue";
      case 58: return "the Robber";
      case 59: return "the Robber";
      case 60: return "the Robber";
      case 61: return "the Robber";
      case 62: return "the Robber";
      case 63: return "the Magswoman";
      case 64: return "the Magswoman";
      case 65: return "the Magswoman";
      case 66: return "the Magswoman";
      case 67: return "the Magswoman";
      case 68: return "the Highwaywoman";
      case 69: return "the Highwaywoman";
      case 70: return "the Highwaywoman";
      case 71: return "the Highwaywoman";
      case 72: return "the Highwaywoman";
      case 73: return "the Burglaress";
      case 74: return "the Burglaress";
      case 75: return "the Burglaress";
      case 76: return "the Burglaress";
      case 77: return "the Burglaress";
      case 78: return "the Thief";
      case 79: return "the Thief";
      case 80: return "the Thief";
      case 81: return "the Thief";
      case 82: return "the Knifer";
      case 83: return "the Knifer";
      case 84: return "the Knifer";
      case 85: return "the Knifer";
      case 86: return "the Quick-Blade";
      case 87: return "the Quick-Blade";
      case 88: return "the Quick-Blade";
      case 89: return "the Murderess";
      case 90: return "the Murderess";
      case 91: return "the Brigand";
      case 92: return "the Brigand";
      case 93: return "the Cut-Throat";
      case 94: return "the Cut-Throat";
      case 95: return "the Implementress";
      /* no one ever thought up these titles 21-30 */
      case LVL_IMMORT: return "the Immortal Assasin";
      case LVL_GOD: return "the Demi Goddess of thieves";
      case LVL_GRGOD: return "the Goddess of thieves and tradesmen";
      default: return "the Thief";
    }
    break;

    case CLASS_WARRIOR:
    switch(level) {
      case  1: return "the Swordpupil";
      case  2: return "the Swordpupil";
      case  3: return "the Recruit";
      case  4: return "the Recruit";
      case  5: return "the Sentress";
      case  6: return "the Sentress";
      case  7: return "the Fighter";
      case  8: return "the Fighter";
      case  9: return "the Fighter";
      case 10: return "the Soldier";
      case 11: return "the Soldier";
      case 12: return "the Soldier";
      case 13: return "the Soldier";
      case 14: return "the Soldier";
      case 15: return "the Warrior";
      case 16: return "the Warrior";
      case 17: return "the Warrior";
      case 18: return "the Warrior";
      case 19: return "the Warrior";
      case 20: return "the Warrior";
      case 21: return "the Warrior";
      case 22: return "the Warrior";
      case 23: return "the Veteran";
      case 24: return "the Veteran";
      case 25: return "the Veteran";
      case 26: return "the Veteran";
      case 27: return "the Veteran";
      case 28: return "the Veteran";
      case 29: return "the Veteran";
      case 30: return "the Veteran";
      case 31: return "the Veteran";
      case 32: return "the Veteran";
      case 33: return "the Swordswoman";
      case 34: return "the Swordswoman";
      case 35: return "the Swordswoman";
      case 36: return "the Swordswoman";
      case 37: return "the Swordswoman";
      case 38: return "the Swordswoman";
      case 39: return "the Swordswoman";
      case 40: return "the Swordswoman";
      case 41: return "the Swordswoman";
      case 42: return "the Swordswoman";
      case 43: return "the Fenceress";
      case 44: return "the Fenceress";
      case 45: return "the Fenceress";
      case 46: return "the Fenceress";
      case 47: return "the Fenceress";
      case 48: return "the Fenceress";
      case 49: return "the Fenceress";
      case 50: return "the Fenceress";
      case 51: return "the Fenceress";
      case 52: return "the Fenceress";
      case 53: return "the Combatess";
      case 54: return "the Combatess";
      case 55: return "the Combatess";
      case 56: return "the Combatess";
      case 57: return "the Combatess";
      case 58: return "the Heroine";
      case 59: return "the Heroine";
      case 60: return "the Heroine";
      case 61: return "the Heroine";
      case 62: return "the Heroine";
      case 63: return "the Myrmidon";
      case 64: return "the Myrmidon";
      case 65: return "the Myrmidon";
      case 66: return "the Myrmidon";
      case 67: return "the Myrmidon";
      case 68: return "the Swashbuckleress";
      case 69: return "the Swashbuckleress";
      case 70: return "the Swashbuckleress";
      case 71: return "the Swashbuckleress";
      case 72: return "the Swashbuckleress";
      case 73: return "the Mercenaress";
      case 74: return "the Mercenaress";
      case 75: return "the Mercenaress";
      case 76: return "the Mercenaress";
      case 77: return "the Mercenaress";
      case 78: return "the Swordmistress";
      case 79: return "the Swordmistress";
      case 80: return "the Swordmistress";
      case 81: return "the Swordmistress";
      case 82: return "the Swordmistress";
      case 83: return "the Lieutenant";
      case 84: return "the Lieutenant";
      case 85: return "the Lieutenant";
      case 86: return "the Lieutenant";
      case 87: return "the Lieutenant";
      case 88: return "the Lady Champion";
      case 89: return "the Lady Champion";
      case 90: return "the Lady Champion";
      case 91: return "the Lady Dragoon";
      case 92: return "the Lady Dragoon";
      case 93: return "the Cavalier";
      case 94: return "the Cavalier";
      case 95: return "the Lady Knight";
      /* no one ever thought up these titles 21-30 */
      case LVL_IMMORT: return "the Immortal Lady of War";
      case LVL_GOD: return "the Queen of Destruction";
      case LVL_GRGOD: return "the Goddess of war";
      default: return "the Warrior";
    }
    break;
  }

  /* Default title for classes which do not have titles defined */
  return "the Classless";
}

